@@ -31,11 +31,11 @@ has 'script'       => is => 'lazy', isa => NonEmptySimpleStr;
 
 # Private methods
 sub _build_owner {
-   return $_[ 0 ]->_inflate_symbol( $_[ 1 ], 'prefix' ) || 'root';
+   return $_[ 0 ]->inflate_symbol( $_[ 1 ], 'prefix' ) || 'root';
 }
 
 sub _build_script {
-   return basename( $_[ 0 ]->_inflate_path( $_[ 1 ], qw( pathname ) ) );
+   return basename( $_[ 0 ]->inflate_path( $_[ 1 ], 'pathname' ) );
 }
 
 1;
@@ -138,7 +138,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2014 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
