@@ -45,7 +45,7 @@ sub get_package_meta {
       $path->exists and return Class::Usul::Response::Meta->new( $path );
    }
 
-   throw PathNotFound, args => [ $file ], level => 3;
+   throw PathNotFound, [ $file ], level => 3;
    return; # Not reached
 }
 
@@ -137,7 +137,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2014 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
